
attribute vec4 a_position;
attribute vec3 a_normal;
attribute vec2 a_texCoord;
attribute vec4 a_color;

#ifdef GL_ES
varying highp	vec4	v_mpos;
varying highp	vec4	v_spos;
varying mediump	vec3	v_normal;
varying lowp	vec4	v_color;
varying mediump	vec2	v_texCoord;
varying mediump	vec4	v_shadowCoord;
varying lowp	vec3	v_fogColor;
varying mediump float	v_fogParam;
#else
varying			vec4	v_mpos;
varying			vec4	v_spos;
varying			vec3	v_normal;
varying			vec4	v_color;
varying			vec2	v_texCoord;
varying			vec4	v_shadowCoord;
varying			vec3	v_fogColor;
varying			float	v_fogParam;
#endif

uniform vec3	u_curveOrigin;
uniform vec3	u_offsetOrigin;
uniform float	u_curveRadius;
uniform float	u_offsetHeight;
uniform float	u_offsetFreq;
uniform mat4	u_shadowMatrix;

void main()
{
	vec3  Rc		= a_position.xyz - u_curveOrigin;
	float radius	= length(Rc);
	float curveX	= Rc.x/u_curveRadius;
	float curveY	= -Rc.y/u_curveRadius;
	float offsetX	= u_offsetFreq*(sin((a_position.x - u_offsetOrigin.x)/2.0));
	float offsetY	= u_offsetFreq*(sin((a_position.y - u_offsetOrigin.y)/3.0));

	vec2 vs			= vec2(sin(curveX),sin(curveY));
	vec2 vc			= vec2(cos(curveX),cos(curveY));
	mat3 Mx			= mat3(	vc.x,	0.0,	vs.x,
							0.0,	1.0,	0.0,
							-vs.x,	0.0,	vc.x );
	
	mat3 My			= mat3(	1.0,	0.0,	0.0,
							0.0,	vc.y,	-vs.y,
							0.0,	vs.y,	vc.y );

	mat3 Mcurve		= Mx * My;
	vec4 wp			= vec4(u_curveOrigin + Mcurve * (vec3(0.0,0.0,Rc.z)),a_position.w) + vec4(0,0,u_offsetHeight*(sin(offsetX) + sin(offsetY)),0);
	gl_Position		= CC_MVPMatrix * wp;

	mat3 normalMat	= mat3(Mcurve) * CC_NormalMatrix;
	v_normal		= normalMat * a_normal;
	v_spos			= gl_Position;
	v_mpos			= CC_MVMatrix * wp;
	//v_mpos			= wp;
	v_color			= a_color;
	v_texCoord		= a_texCoord;
	//v_shadowCoord	= u_shadowMatrix * a_position;
	v_shadowCoord	= u_shadowMatrix * wp;
}