
#ifdef GL_ES

precision mediump float;

varying highp	vec4 v_mpos;
varying mediump	vec3 v_normal;
varying lowp	vec4 v_color;
varying mediump	vec2 v_texCoord;
varying mediump	vec4 v_shadowCoord;
varying lowp	vec3 v_fogColor;
varying mediump float v_fogParam;
#else
varying vec4	v_mpos;
varying vec3	v_normal;
varying vec4	v_color;
varying vec2	v_texCoord;
varying vec4	v_shadowCoord;
varying vec3	v_fogColor;
varying float	v_fogParam;
#endif

uniform vec3	u_lightDir;
uniform vec3	u_lightAmbient;
uniform vec3	u_lightDiffuse;
uniform vec3	u_lightSpecular;
uniform float	u_specOpacity;
uniform float	u_specPower;
uniform float	u_glowOpacity;

uniform vec3	u_fogA;
uniform vec3	u_fogB;
uniform vec3	u_fogC;
uniform float	u_fogMin;
uniform float	u_fogMax;
uniform float	u_fogHeight;

uniform sampler2D	u_shadowMap;
uniform sampler2D	u_glowMap;

void main()			
{
	vec3 eye		= normalize(v_mpos.xyz)*-1.0;
	vec3 N			= normalize(v_normal);
	float eyeDot	= max(dot(N,eye),0.0);

	vec4 cDiffuse	= texture2D(CC_Texture0,v_texCoord);
	vec3 cGlow		= (v_color.rgb * texture2D(u_glowMap,v_texCoord).rgb) * u_glowOpacity;
	
	float refDot	= max(dot(reflect(u_lightDir,N),eye),0.0);
	vec3 cSpec		= u_lightSpecular.rgb*( pow( refDot, u_specPower ) );

	float lightDot	= clamp( dot( N,u_lightDir )*-1.0, 0.0, 1.0 );
	
	/*
	float bias		= clamp( 0.005*tan(acos(lightDot)), 0.0, 0.01 );
	vec4 shadowTP	= clamp( (v_shadowCoord / v_shadowCoord.w)*vec4(0.5) + vec4(0.5), vec4(0.0), vec4(1.0) );
	float zMap		= texture2D(u_shadowMap,shadowTP.xy).z;
	float shadow	= 1.0 - 0.8*step(zMap,shadowTP.z-bias);
	*/

	//vec3 c2			= min( v_color.rgb*cDiffuse.rgb*(u_lightAmbient + u_lightDiffuse*(lightDot*shadow)) + (cSpec)*(u_specOpacity*shadow) + cGlow, vec3(1.0) );
	vec3 c2			= min( v_color.rgb*cDiffuse.rgb*(u_lightAmbient + u_lightDiffuse*(lightDot)) + (cSpec)*(u_specOpacity) + cGlow, vec3(1.0) );

	/*
	float fogParam	= clamp((length(v_mpos.xyz)-u_fogMin)/(u_fogMax-u_fogMin),0.0,1.0);
	float tFog		= clamp(v_mpos.y/u_fogHeight + 0.5,0.0,1.0);
	float tFogA		= smoothstep(0.0,1.0,min(tFog/0.5,1.0));
	float tFogB		= smoothstep(0.0,1.0,max((tFog-0.5)/0.5,0.0));
	
	vec3 fogB2		= mix(u_fogB,u_fogC,pow(fogParam,6.0));
	vec3 fogColor	= mix(mix(u_fogA,fogB2,tFogA),u_fogB,tFogB);
	*/

	//gl_FragColor	= vec4(mix(c2,fogColor,smoothstep(0.0,1.0,fogParam)),v_color.a*cDiffuse.a);
	gl_FragColor	= vec4(c2,v_color.a*cDiffuse.a);
}				
